# Integración de Laravel con Chatbot de Rasa

## Descripción
Este documento sirve como una guía y registro del proceso de integración de un
módulo en Laravel para interactuar con un chatbot de Rasa a través de su API
REST.


## 1 API de Rasa
La integración con el chatbot de Rasa se realiza a través de su API REST, la
cual permite enviar mensajes y recibir respuestas del bot. Es importante
familiarizarse con la documentación oficial para comprender cómo interactuar con
el bot a través de esta API.

### Documentación Oficial de Rasa

La documentación de la API REST de Rasa se encuentra disponible en:
[https://rasa.com/docs/rasa/pages/http-api](https://rasa.com/docs/rasa/pages/http-api)



### Iniciar una Conversación

El inicio de una conversación con el chatbot de Rasa se realiza mediante el
envío de un mensaje al tracker del bot. Esto se logra a través de una petición
POST al endpoint `/conversations/{conversation_id}/messages`, donde
`{conversation_id}` es un identificador único para la conversación o el usuario.



#### Ejemplo de Petición

```json

POST /conversations/{conversation_id}/messages

Content-Type: application/json

{
    "text": "Hello!",
    "sender": "user"
}
```

#### Respuesta

La respuesta a esta petición incluye el tracker actualizado, del cual se puede
extraer el mensaje de respuesta del bot. Este tracker es crucial para entender
el contexto de la conversación y determinar si la respuesta del bot es la
adecuada.

#### Predicción del resultado

Esta funcionalidad aún está por explorar.

Antes de realizar acciones, es posible considerar el uso del endpoint
/conversations/{conversation_id}/predict para predecir el próximo resultado de
la conversación. Esto puede ser útil para anticipar las respuestas del bot y
ajustar la interacción en consecuencia.


#### Notas Importantes

    Es vital generar un conversation_id único para cada usuario o conversación
    para mantener un seguimiento adecuado.  Familiarizarse con la estructura del
    tracker y cómo interpretar sus datos es fundamental.
    
## 2 Preparación del Entorno de Laravel
Para el desarrollo del módulo que permitirá la comunicación con el chatbot de
Rasa, es posible utilizar Laravel o su versión más ligera, Lumen. La elección
entre Laravel y Lumen dependerá de las necesidades específicas del proyecto y de
las preferencias personales. Laravel ofrece una solución más completa con todas
las características de un framework de desarrollo web de PHP, mientras que Lumen
es una opción más ligera y rápida, ideal para microservicios y APIs.

### Configuración del Entorno:

1. **Instalación de Laravel o Lumen**:
   Asegúrate de tener Composer instalado en tu sistema. Luego, crea un nuevo
   proyecto utilizando el comando de Composer correspondiente al framework
   elegido.
   - Para Laravel:
     ```
     composer create-project --prefer-dist laravel/laravel nombreProyecto
     ```
   - Para Lumen:
     ```
     composer create-project --prefer-dist laravel/lumen nombreProyecto
     ```

  Tras esto habrá que configurar el .env de manera conveniente según todo el resto
  del entorno. En el caso típico implica configurar la conexión a la base de
  datos y migración de las tablas iniciales. También se incluyen algunos puntos
  de configuración más en pasos posteriores.

2. **Instalación del Código como Módulo**:
   Una vez establecido el entorno básico de Laravel o Lumen, el siguiente paso
   es integrar el módulo de comunicación con el chatbot de Rasa. Esto se hará
   mediante el uso de Composer y un repositorio de GitLab donde residirá el
   módulo.

   - Configura el repositorio de GitLab como un paquete Composer. Esto implica
   añadir el repositorio a tu archivo `composer.json` y luego requerir el
   paquete como cualquier otra dependencia. Por ejemplo:
     ```json
     "repositories": [
       {
         "type": "vcs",
         "url": "<url-del-repositorio>"
       }
     ],
     "require": {
       "aunnait/rasa-alicante": "dev-master"
     }
     ```
   - Ejecuta `composer update` para instalar el módulo en tu proyecto.

Para **desarrollos en local**, para poder actualizar cambios desde la carpeta donde
está el módulo en lugar de tener que pasar por todo el ciclo de Git de commits
subidas y despliegues, se puede realizar la siguiente modificación en el archivo
`composer.json`:

     ```json
     "repositories": [
       {
         "type": "path",
         "url": "/ruta/al/repositorio"
       }
     ],
     "require": {
       "aunnait/rasa-alicante": "dev-master"
     }
     ```
 
**3. Configuración del entrenamiento diario**

Para realizar el entrenamiento diario se tiene que añadir el trabajo al cron de
Laravel. Para ello hay que editar el archivo *config/Kernel.php* de la instalación de
Laravel y añadir una línea en el comando *schedule(Schedule $schedule)*

```php
protected function schedule(Schedule $schedule)
{
    $schedule->command('chatbot:train')->dailyAt('03:00'); 
}
```

También **IMPORTANTE Y OBLIGATORIO** confirmar que el cron está activo en
Laravel, para ello hay que tener presente que en el cron del sistema operativo
debe existir una línea que llame al cron de laravel cada minuto, de esta manera:

```
* * * * * cd /var/www/web && php artisan schedule:run >> /dev/null 2>&1
```

**4. Configuración de las colas asíncronas**
Hay que asegurarse que las colas sean asíncronas para poder realizar las tareas como
el entrenamiento y otras que puedan surgir de tal forma. Para ello se debe cambiar
el QUEUE_CONNECTION en el .env de la siguiente forma:

```
# Archivo .env
QUEUE_CONNECTION=database
```

Después hay que crear la tabla de colas y hacer la migración

```bash
php artisan queue:table
php artisan migrate
```

Para asegurar la ejecución de las colas se debe de copiar el archivo 
*templates/laravel-queues.service* a */etc/systemd/system* y después ejecutar como root
```bash
systemctl daemon-reload
systemctl enable laravel-queues
systemctl start laravel-queues
```
**Es IMPORTANTE señalar** que en ese archivo se presupone la web instalada en la
ruta */var/www/web*, si fuera cualquier otra, habría que cambiarla previamente a su copia.

## 3 Creación del Módulo en Laravel
El módulo destinado a la comunicación con el chatbot de Rasa fue desarrollado
siguiendo un enfoque modular para tratar de dar una separación de
responsabilidades.

### Estructura del Módulo:
El módulo fue estructurado de manera que cada elemento se sitúa de forma lógica
y coherente dentro del proyecto. Todos los archivos se encuentran bajo el
subdirectorio src. 

- **RasaServiceProvider.php**: Configura la librería e indica los archivos que deben de
ser cargados.

- **Controladores**: Se deja la carpeta `Controllers` preparada para el caso de
necesitar crear endpoints.

- **Rutas**: Se deja la carpeta `routes` preparada en caso de necesitar crear
rutas para posibles controladores.

- **Configuración**: Se añadió un archivo de configuración en `config/RasaConfig.php`
para mantener variables relevantes como la URL base del API de Rasa y cualquier
otro parámetro de configuración necesario para la integración. 

- **events**: Carpeta que contiene el evento `RasaRespuesta`. Este evento se
lanzará cuando se envíe un texto y se obtenga la respuesta. La idea es permitir
que el resto del sistema pueda estar al tanto de respuestas de conversaciones
con el fin de almacenarlas en base de datos o darle un tratamiento que puedan
requerir.

- **RasaComm.php**: Clase para la interacción con la aplicación. Es la que se
importará desde cualquier punto de Laravel y permitirá el envío de mensajes. Es
donde se deben de implementar todos los métodos relativos a la comunicación
saliente.

## 4. Requisitos previos

Para el correcto funcionamiento de la aplicación, deben de realizarse unas configuraciones
previas que permitan el uso de Rasa por parte de la aplicación web. Para ello se deberá:

1. Confirmar el uso de SystemD para poder lanzar Rasa como servicios del sistema
1. Asegurarse de tener una instalación de python válida, recomendable la versión 3.10
1. Crear el directorio /opt/rasa donde se encontrarán todo el código y el mismo Rasa
1. Realizar una instalación de un virtual env de python en ese directorio
1. Dar los permisos adecuados para el usuario correspondiente
1. Copiar el archivo *scripts/templates/sudo.rasa* a */etc/sudoers.d/rasa*
1. Copiar el archivo *scripts/templates/rasa@.service* a */etc/systemd/system*
1. Copiar el archiov *scripts/tempaltes/laravel.cron* a */etc/cron.d/laravel*

Para realizar los pasos anteriores se puede utilizar el siguiente código, estando
logueado como el usuario root y en el directorio del paquete:

```bash
# Crear directorio
mkdir -p /opt/rasa

# Crear entorno virtual
python3 -m venv /opt/rasa

# Instalar rasa e idiomas
/opt/rasa/bin/pip install rasa rasa[spacy]
/opt/rasa/bin/python -m spacy download es_core_news_md
/opt/rasa/bin/python -m spacy download en_core_web_md
/opt/rasa/bin/python -m spacy download ca_core_news_md

# Instalar servidor de acciones y websockets
mkdir -p /opt/rasa/action
/opt/rasa/bin/python -m rasa init --no-prompt --init-dir /opt/rasa/action
cp scripts/templates/actions.py /opt/rasa/action/actions
cp scripts/templates/run-action.sh /opt/rasa/action
cp scripts/templates/rasa-action.service /etc/systemd/system
cp scripts/templates/laravel-websocket.service /etc/systemd/system
systemctl daemon-reload
systemctl enable rasa-action
systemctl start rasa-action
systemctl enable laravel-websocket
systemctl start laravel-websocket

# Permisos usuario
chown www-data.www-data -R /opt/rasa

# copiar archivos
cp scripts/templates/sudo.rasa /etc/sudoers.d/rasa
cp scripts/templates/rasa@.service /etc/systemd/system
systemctl daemon-reload
```


## 5. Configuración del paquete

Agrega la siguiente línea al final de tu archivo .env. Esta variable almacenará la URL
 de tu servidor de Rasa:
```arduino
RASA_URL=http://tu_servidor_rasa
```

Para entornos de desarrollo local, se puede utilizar la siguiente dirección de prueba:

```arduino
RASA_URL=http://alicanteweb.aunnaitapp.es
```

Hay que reemplazar http://tu_servidor_rasa con la URL específica del servidor de Rasa 
cuando se esté configurando entornos de producción o de prueba diferentes.



## 6. Uso del paquete
Para la implementación de la comunicación entre el software desarrollado y el
chatbot de Rasa, se estableció un método específico para el envío de mensajes,
evitando el uso directo de endpoints. Este enfoque garantiza que el paquete
pueda procesar y enviar los mensajes recibidos desde la interfaz de usuario del
chat hacia el chatbot de Rasa de manera eficiente.

### Crear un bot:
Los nuevos bots se crearán con el uso del comando *createBot($id,$idioma,$puerto)* de la
clase ***\Aunnait\Rasalicante\RasaBotControl***.

- El parámetro *id* será el identificador del bot, y se utilizará, entre otras 
  cosas, para crear la carpeta relacionada con el mismo.
- El parámetro *$idioma* indicará el idioma a utilizar por el bot. Es necesario
  para la configuración inicial del mismo.
- El parámetro *$puerto* indicará en que puerto debe de escuchar este bot.

### Eliminar un bot:
Para esto sólo será necesario utilizar el comando *deleteBot($id)* indicando el *$id*
del bot que se quiere eliminar. Será detenido y borrado.

### Reiniciar un bot:
De manera similar al método anterior, sólo será necesario el uso del comando
*rebootBot($id)* indicando su *$id* correspondiente.

### Comprobar el estado de un bot:
Para comprobar el estado en el que están los bots de un mismo idioma se vale del uso
del comando *statusBot($id)* indicando su *$id* correspondiente.

El resultado será un array con los idiomas disponibles del bot y el estado de cada
uno que pueden ser *active* cuando es correcto y *failed* u otro estado si no se
está ejecutando.

### Entrenar un bot:
Al igual que con los elementos anteriores, para entrenar un modelo se necesitará 
utilizar el método entrena($bot, $language) de ***RasaBotControl***. El puerto a 
utilizar será consultado en el mismo método.


### Envío de Mensajes:
El envío de mensajes se realiza a través del objeto
`Aunnait\Rasalicante\RasaComm` y su método `enviarMensaje($idBot, $language, $usuario, $mensaje)`.
Este método facilita la comunicación con el chatbot, donde:

- `$idBot` es el identificador del bot al que se va a realizar el envío del texto de
la conversación.

- `$language` es el idioma en el que se va a enviar el mensaje. Es necesario para
configurar el bot y asegurar que el mensaje se procese correctamente.

- `$usuario` es el identificador único de la comunicación, necesario para
mantener un diálogo coherente y continuo con el chatbot. Cada sesión de chat con
un usuario debe tener un `$usuario` único para asegurar que la conversación se
maneje de forma individual.

- `$mensaje` es el texto enviado por el usuario, que se desea comunicar al
chatbot de Rasa para su procesamiento.

### Respuesta y Evento:
La función `enviarMensaje` devuelve una respuesta en formato JSON con la estructura:
```json
{
  "usuario": "usuario utilizado para la comunicacion",
  "texto": "Texto de la contestación",
  "intencion": "Intencion identificada por rasa",
  "form": "Nombre dle formulario activo",
  "slots": {"slot1":"valor"}
}
```

Los objetos de ***slots*** son los parámetros de los formularios. Se irán rellenando 
a medida que se vayan cumplimentando. 

El parámetro ***form*** indica si hay un formulario activo y cual es, para saber
que se encuentra activo y en bucle del mismo.

Esta respuesta no solo proporciona el texto de la contestación del chatbot, sino
que también incluye la intención identificada por Rasa, lo cual es crucial para
entender el propósito detrás del mensaje del usuario.

Adicionalmente, al enviar un mensaje y recibir una respuesta, se genera un
evento del tipo `Aunnait\Rasalicante\Events\RasaRespuesta`. Este evento contiene
los campos usuario, texto, e intencion, reflejando así toda la información
relevante de la comunicación con el chatbot. Este mecanismo de eventos es
particularmente útil para sistemas que requieren procesar o actuar sobre la
información de la conversación en otros componentes o servicios dentro de la
aplicación.


### Usar NLU para clasificar

Se puede utilizar la función `testNLU` de la clase ***\Aunnait\Rasalicante\RasaComm***
para enviar un texto a un bot y sa er la intención con la que lo clasificaría y
los conceptos que encontraría en el mismo.

Este método se utiliza con los parámetros:

- `$bot` es el identificador del bot al que se va a realizar el envío del texto de
la conversación.

- `$language` es el idioma en el que se va a enviar el mensaje. Es necesario para
configurar el bot y asegurar que el mensaje se procese correctamente.

- `$text` es el texto que se va a enviar el bot para su analisis NLU

La respuesta de esta función será un json con el siguiente formato:

```json
{
  "intencion": "Intencion identificada por rasa",
  "porcentaje": "Nombre dle formulario activo",
  "slots": [{"nombre":"valor"}]
}
```

En esta respuesta `intencion` es el nombre de la intención con la que asociará el
texto. `porcentaje` representa el porcentaje (entre 0 y 1) de seguridad que tiene con
la asociación de esa intención.

Finalmente `slots` será un array de los conceptos localizados en el texto, cada
elemento de este array será el nombre del concepto encontrado y su valor. Por ejemplo
en una intención en la que se estuvieran dando nombres de municipios aquí podría
encontrarse algo como ```"slots":[{"municipio":"Murcia"}]```

## 7. Integración con el Sistema:

El diseño de esta estructura de envío y respuesta asegura una integración fluida
y flexible con aplicaciones que utilizan el paquete. Al centralizar la
comunicación con el chatbot a través de un método específico y generar eventos
con la información relevante, se facilita la expansión y adaptabilidad del
sistema para futuros desarrollos o necesidades operativas.

