<?php

namespace Aunnait\Rasalicante;
use Symfony\Component\Yaml\Yaml;
use Illuminate\Support\Facades\Log;

class RasaBotControl {


    /**
     * Método para instalar las dependencias de rasa
     *
     * @return void
     */
    public function installDependencies() {

        // Crea un entorno virtual de python en /opt/rasa
        exec('python3 -m venv /opt/rasa', $output, $return_var);
        if ($return_var != 0) {
            throw new \Exception('Error al crear el entorno virtual de python');
        }

        // Ejecuta el comandonn "pip install rasa" para instalar rasa utilizando el entorno virtual de /opt/rasa
        exec('/opt/rasa/bin/pip install rasa rasa[spacy]', $output, $return_var);
        if ($return_var != 0) {
            throw new \Exception('Error al instalar rasa');
        }

        // Ejecuta el comando "python -m spacy download es_core_news_md" para instalar el modelo de lenguaje español
        exec('/opt/rasa/bin/python -m spacy download es_core_news_md', $output, $return_var);
        if ($return_var != 0) {
            throw new \Exception('Error al instalar el modelo de lenguaje español');
        }

        // Instala el modelo de spacy en inglés
        exec('/opt/rasa/bin/python -m spacy download en_core_web_md', $output, $return_var);
        if ($return_var != 0) {
            throw new \Exception('Error al instalar el modelo de lenguaje inglés');
        }

        // Instala el modelo de spacy en catalan
        exec('/opt/rasa/bin/python -m spacy download ca_core_news_md', $output, $return_var);
        if ($return_var != 0) {
            throw new \Exception('Error al instalar el modelo de lenguaje catalán');
        }

    }



    /**
     * Crea un nuevo bot con los parámetros especificados.
     *
     * @param int $id El ID del bot.
     * @param string $language El lenguaje del bot.
     * @param int $puerto El puerto del bot.
     * @return void
     */
    public function createBot($id, $language, $puerto) {
        //$id y $puerto no pueden ser nulos o vacios
        if (is_null($id) || $id == "") {
            throw new \Exception('El id no puede ser nulo o vacio');
        }
        if (is_null($puerto) || $puerto == "") {
            throw new \Exception('El puerto no puede ser nulo o vacio');
        }

        $id = $id.'-'.$language;
        $path = '/opt/rasa/' . $id;
        if (!file_exists($path)) {
            mkdir($path, 0777, true);
        }

        // Ejecuta el comando "rasa init" sin dialogo para crear el bot en ese directorio
        exec('cd '.$path.'; LOG_LEVEL=DEBUG /opt/rasa/bin/python -m rasa init --no-prompt', $output, $return_var);
        if ($return_var != 0) {
            throw new \Exception('Error al crear el bot');
        }

        // copiar run.sh y darle +x al directorio del bot
        copy(__DIR__ . '/../scripts/templates/run.sh', $path . '/run.sh');
        chmod($path . '/run.sh', 0777);

        // copiar accent_normalizer.py  al directorio del bot
        copy(__DIR__ . '/../scripts/templates/accent_normalizer.py', $path . '/accent_normalizer.py');


        // cambia el string *puerto* por $puerto en el archivo run.sh
        $file = file_get_contents($path . '/run.sh');
        $file = str_replace('*puerto*', $puerto, $file);
        file_put_contents($path . '/run.sh', $file);

        // Configurar el endpoint de las acciones del bot, que estará en el mismo
        // servidor que el bot, en el puerto 5055
        $endpoints = [
            'action_endpoint' => [
                'url' => 'http://localhost:5055/webhook'
            ]
        ];
        $yaml = Yaml::dump($endpoints, 4, 2);
        file_put_contents($path . '/endpoints.yml', $yaml);

        // Crea un servicio de sistema en systemd para el usuario para lanzar el bot
        exec('sudo systemctl enable rasa@' . $id . '.service', $output, $return_var);
        exec('sudo systemctl start rasa@' . $id . '.service', $output, $return_var2);

        if ($return_var != 0 || $return_var2 != 0) {
            throw new \Exception('Error al crear el servicio de sistema');
        }

    }


    /**
     * Elimina un bot.
     *
     * @param int $id El ID del bot a eliminar.
     * @param string $language El lenguaje del bot.
     * @return void
     */
    public function deleteBot($id, $language=null) {

        // comprobar que id no sea nulo o vacio ("")
        if (is_null($id) || $id == "") {
            throw new \Exception('El id no puede ser nulo o vacio');
        }

        $languages=array();
        if ( $language == "") {
            throw new \Exception('El lenguaje no puede ser vacio');
        } elseif (is_null($language)) {
            // Get all the languages for teh bot
            $langs = \DB::table('chatbot_port')->where('chatbots_id', $id)->select('language')->get();
            foreach ($langs as $lang) {
                $languages[] = $lang->language;
            }
        } else {
            $languages[] = $language;
        }

        foreach ($languages as $language) {
            $id = $id.'-'.$language;

            // Elimina el servicio de sistema en systemd
            exec('sudo systemctl disable rasa@' . $id . '.service', $output, $return_var);
            exec('sudo systemctl stop rasa@' . $id . '.service', $output, $return_var2);

            $path = '/opt/rasa/' . $id;
            if (file_exists($path)) {
                exec('sudo rm -rf ' . $path);
            }

            if ($return_var != 0 || $return_var2 != 0) {
                throw new \Exception('Error al borrar el bot');
            }
        }

    }


    /**
     * Reinicia el bot especificado.
     *
     * @param string $bot El nombre del bot a reiniciar.
     * @return void
     */
    public function rebootBot($bot, $language = null) {
        $languages = \DB::table('chatbot_port')->where('chatbots_id', $bot)->select('language')->get();
        foreach ($languages as $lang) {
            if ( is_null(($language)) || $lang->language == $language) {
                $bot = "$bot-" . $lang->language;
                exec('sudo systemctl restart rasa@' . $bot . '.service', $output, $return_var);
                if ($return_var != 0) {
                    throw new \Exception('Error al reiniciar el bot');
                }
            }
        }

    }


    /**
     * Detiene el bot especificado.
     *
     * @param string $bot El nombre del bot a detener.
     * @return void
     */
    public function stopBot($bot, $language = null) {
        $languages = \DB::table('chatbot_port')->where('chatbots_id', $bot)->select('language')->get();
        foreach ($languages as $lang) {
            if ( is_null(($language)) || $lang->language == $language) {
                // Exec stop service and get result code
                exec('sudo systemctl stop rasa@' . $bot . '-' . $lang->language . '.service', $output, $return_var);
                if ($return_var != 0) {
                       throw new \Exception('Error al parar el bot');
                }
            }
        }

    }

    /**
     * Inicia el bot especificado.
     *
     * @param string $bot El nombre del bot a iniciar.
     * @return void
     */
    public function startBot($bot, $language = null) {
        $languages = \DB::table('chatbot_port')->where('chatbots_id', $bot)->select('language')->get();
        foreach ($languages as $lang) {
            if ( is_null(($language)) || $lang->language == $language) {
                // Comprobar si el bot está parado
                $status = exec('sudo systemctl is-active rasa@' . $bot . '-' . $lang->language . '.service', $output, $return_var);
                if ($return_var === 3) {
                    // Si está parado, arrancarlo
                    exec('sudo systemctl start rasa@' . $bot . '-' . $lang->language . '.service', $output, $return_var);
                } elseif ($return_var != 0) {
                    throw new \Exception('Error al arrancar el bot');
                }
            }
        }


    }

    /**
     * Devuelve el estado del bot especificado.
     *
     * @param string $bot El nombre del bot.
     * @return string El estado del bot.
     */
    public function statusBot($bot) {
        $languages = \DB::table('chatbot_port')->where('chatbots_id', $bot)->select('language')->get();
        $status = [];
        foreach ($languages as $lang) {
            $status[$lang->language] = exec('sudo systemctl is-active rasa@' . $bot . '-' . $lang->language . '.service');
        }
        return $status;

    }



    /**
     * Entrena el bot con el nombre y el idioma especificados.
     *
     * @param string $bot El nombre del bot.
     * @param string $language El idioma en el que se entrenará el bot.
     * @return string El resultado del entrenamiento.
     */
    public function entrena(String $bot, String $language, String $callbackurl=null) {
        Log::debug("El entrnamiento para el bot $bot en $language va a comenzar");
        $rt = new RasaTraining();
        $basePath = "/opt/rasa/$bot-$language/";

        $port = \DB::table('chatbot_port')->where('chatbots_id', $bot)->where('language', $language)->first()->port;
        if ($port == null) {
            throw new \Exception("No se ha encontrado el puerto para el bot $bot y el idioma $language");
        }

        // Crear domain.yml
        $domain = $rt->createDomainYaml($bot, $language);

        // Crear nlu.yml
        $nlu = $rt->createNluYaml($bot, $language);

        // Crear stories.yml
        $stories = $rt->createStoriesYaml($bot,$language);


        // Crear rules.yml
        $rules = $rt->createRulesYaml($bot, $language);

        // Crear pipeline.yml
        $pipeline = $rt->createPipelineYaml($bot, $language);

        // Crear rulespolicy.yml
        $rulespolicy = $rt->createRulesPolicyYaml($bot, $language);
        // Write the rulespolicy to /tmp/bot-{botid}-rules.yml

        // Escribir el contenido de $rules en /opt/rasas/catalan/data/rules.yml
        $file_path = $basePath.'data/rules.yml';
        file_put_contents($file_path, $rules);
        // Escribir el contenido de $stories en /opt/rasa/catalan/data/stories.yml
        $file_path = $basePath.'data/stories.yml';
        file_put_contents($file_path, $stories);

        // Escribir el contenido de $nlu en /opt/rasa/catalan/data/nlu.yml
        $file_path = $basePath.'data/nlu.yml';
        file_put_contents($file_path, $nlu);

        // Escribir el contenido de $domain en /opt/rasa/catalan/domain.yml
        $file_path = $basePath.'domain.yml';
        file_put_contents($file_path, $domain);

        // Escribir el contenido de $pipeline en /opt/rasa/catalan/config.yml
        $file_path = $basePath.'config.yml';
        file_put_contents($file_path, $pipeline);

        // Append el contenido de $rulespolicy en /opt/rasa/catalan/config.yml
        file_put_contents($file_path, $rulespolicy, FILE_APPEND);

        $data = [
            $domain,
            $pipeline,
            $nlu,
            $stories,
            $rules,
            $rulespolicy
        ];

        //$rasaEndpoint = "http://127.0.0.1:5005/model/train";
        //$rasaEndpoint = config('rasa.rasa_url') . ":$port/model/train";
        $rasaEndpoint = "http://127.0.0.1:$port/model/train";
        if ($callbackurl != null) {
            $rasaEndpoint .= "?callback_url=$callbackurl";
        }
        $data_string=implode("\n",$data);
        $ch = curl_init($rasaEndpoint);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
        curl_setopt($ch, CURLOPT_POSTFIELDS, $data_string);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            'Content-Type: application/yaml')
        );
        $respuesta = curl_exec($ch);

        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        if ($httpCode != 200) {
            throw new \Exception("Error al entrenar el bot $bot en el puerto $port. Código de respuesta: $httpCode. Respuesta: $respuesta");
        }

        Log::debug("El entrenamiento para el bot $bot en $language finalizo con código $httpCode");
        return $httpCode;
    }


}
